Shader "Project2/SkyboxStereo" {
Properties {
    _Tint ("Tint Color", Color) = (.5, .5, .5, .5)
    [Gamma] _Exposure ("Exposure", Range(0, 8)) = 1.0
    _Rotation ("Rotation", Range(0, 360)) = 0
    [NoScaleOffset] _TexLeft ("Left Cubemap", Cube) = "grey" {}
		[NoScaleOffset] _TexRight ("Right Cubemap", Cube) = "grey" {}
}

SubShader {
    Tags { "Queue"="Background" "RenderType"="Background" "PreviewType"="Skybox" }
    Cull Off ZWrite Off

    Pass {

        CGPROGRAM
        #pragma vertex vert
        #pragma fragment frag
        #pragma target 2.0

        #include "UnityCG.cginc"

        samplerCUBE _TexLeft;
				samplerCUBE _TexRight;
        half4 _TexLeft_HDR;
		half4 _TexRight_HDR;

        half4 _Tint;
        half _Exposure;
        float _Rotation;

        uniform int _RenderingMode;

        float3 RotateAroundYInDegrees (float3 vertex, float degrees)
        {
            float alpha = degrees * UNITY_PI / 180.0;
            float sina, cosa;
            sincos(alpha, sina, cosa);
            float2x2 m = float2x2(cosa, -sina, sina, cosa);
            return float3(mul(m, vertex.xz), vertex.y).xzy;
        }

        struct appdata_t {
            float4 vertex : POSITION;
            UNITY_VERTEX_INPUT_INSTANCE_ID
        };

        struct v2f {
            float4 vertex : SV_POSITION;
            float3 texcoord : TEXCOORD0;
            UNITY_VERTEX_OUTPUT_STEREO
        };

        v2f vert (appdata_t v)
        {
            v2f o;
            UNITY_SETUP_INSTANCE_ID(v);
            UNITY_INITIALIZE_VERTEX_OUTPUT_STEREO(o);
            float3 rotated = RotateAroundYInDegrees(v.vertex, _Rotation);
            o.vertex = UnityObjectToClipPos(rotated);
            o.texcoord = v.vertex.xyz;
            return o;
        }

        fixed4 frag (v2f i) : SV_Target
        {
            UNITY_SETUP_STEREO_EYE_INDEX_POST_VERTEX(i);

            half4 tex;
            half3 c;

            switch (_RenderingMode) {
            default:
            case 0: // Stereo
                if (unity_StereoEyeIndex == 0) { // Left
                    tex = texCUBE(_TexLeft, i.texcoord);
                    c = DecodeHDR(tex, _TexRight_HDR);
                }
                else { // Right
                    tex = texCUBE(_TexRight, i.texcoord);
                    c = DecodeHDR(tex, _TexRight_HDR);
                }
                break;
            case 1: // Mono
                tex = texCUBE(_TexLeft, i.texcoord);
                c = DecodeHDR(tex, _TexRight_HDR);
                break;
            case 2: // LeftOnly
                if (unity_StereoEyeIndex == 0) { // Left
                    tex = texCUBE(_TexLeft, i.texcoord);
                    c = DecodeHDR(tex, _TexRight_HDR);
                }
                else {
                    return half4(0, 0, 0, 0);
                }
                break;
            case 3: // RightOnly
                if (unity_StereoEyeIndex == 1) { // Right
                    tex = texCUBE(_TexRight, i.texcoord);
                    c = DecodeHDR(tex, _TexRight_HDR);
                }
                else {
                    return half4(0, 0, 0, 0);
                }
                break;
            }

            c = c * _Tint.rgb * unity_ColorSpaceDouble.rgb;
            c *= _Exposure;
            return half4(c, 1);
        }
        ENDCG
    }
}
Fallback Off
}